@@ -6,6 +6,8 @@ import getpass
 import pandas as pd
 import os
 import functools
+import webbrowser
+from typing import List
 
 def token_required(f):
     @functools.wraps(f) 
@@ -31,9 +33,12 @@ class ChainBreakerClient():
         try:
             res = requests.get(self._endpoint + "/api/status").status_code
             if res == 200:
-                return "Endpoint is online"
+                print("Endpoint is online")
+                return 200 #"Endpoint is online"
         except: 
-            return "Endpoint is offline. Check our website for more information."
+            pass 
+        print("Endpoint is offline. Check our website for more information.")
+        return 400
 
     def enter_password(self):
         """
@@ -47,6 +52,19 @@ class ChainBreakerClient():
             return self.enter_password()
         return new_password
 
+    def register(self):
+        """
+        This functions lets users to register to ChainBreaker.
+        """
+        name = input("Enter your name: ")
+        email = input("Enter your email: ")
+        data = {"name": name, "email": email}
+        res = requests.put(self._endpoint + "/api/user/register", data)
+        if res.status_code == 200:
+            return "Your account has been created succesfully. Your current permission is 'reader'. If you require more advanced permissions please write us an email to chainbreakerinfo@gmail.com."
+        else:
+            return "Error. Your account has not been created. Please try later or contact us to chainbreakerinfo@gmail.com."
+
     def login(self, *args):
         """
         This function lets the user to connect to ChainBreaker Service.
@@ -112,23 +130,6 @@ class ChainBreakerClient():
             res = requests.put(self._endpoint + "/api/user/change_password", data = data, headers = headers).json()["message"]
             return res
         return "You are logged into your account. Use this function only if you forgot your password and you are not logged into your account."
-
-    @token_required        
-    def create_user(self):
-        """
-        This functions allows administrators to create new users.
-        """
-        if  self._permission == "admin":
-            name = input("User name: ")
-            email = input("User email: ")
-            permission = input("User permission: ")
-            
-            headers = {"x-access-token": self._token}
-            data = {"name": name, "email": email, "permission": permission}
-            res = requests.put(self._endpoint + "/api/user/create_user", data = data, headers = headers).json()["message"]
-            return res
-        else: 
-            print("Only administrators can execute this function.")
             
     @token_required     
     def get_account_info(self):
@@ -142,10 +143,22 @@ class ChainBreakerClient():
         print("Permission: ", self._permission)
 
     @token_required
-    def get_sexual_ads(self, data_version = "1", language = "", website = "", start_date = "0001-01-01", end_date = "9999-01-01", shop_and_services = False): #, features = True, locations = False, comments = False, emails = False, names = False, phone = False, whatsapp = False):
+    def get_df(self, info, reduced = False):
+        df = pd.DataFrame(info)
+        columns = ["id_ad", "data_version", "author", "language", "link", "id_page", "title", "text", "category", 
+        "first_post_date", "extract_date", "website", "phone", "country", "region", "city", "place", "latitude", 
+        "longitude", "zoom", "email", "verified_ad", "prepayment", "promoted_ad", "external_website", "reviews_website", 
+        "nationality", "age", "score_risk"]
+        if reduced: 
+            columns = ["id_ad", "language", "title", "text", "category", "country", "city", "external_website"]
+        df = df[columns]
+        df.set_index("id_ad", inplace = True)
+        return df
+
+    @token_required
+    def get_sexual_ads(self, language = "", website = "", start_date = "0001-01-01", end_date = "9999-01-01"): #, features = True, locations = False, comments = False, emails = False, names = False, phone = False, whatsapp = False):
         """
         This function returns sexual ads data from ChainBreaker Database.
-        - data_version: 1. For more information about data versioning, check ChainBreaker website.
         - language can be: "spanish", "english" or "" (all).
         - website can be: 
           - "mileroticos", "skokka" or "" (all) (for "spanish")
@@ -153,21 +166,11 @@ class ChainBreakerClient():
         - start_date: String in %Y-%m-%d format. Example: 2021-04-28. Default value: 0001-01-01
         - end_date: String in %Y-%m-%d format. Example: 2022-08-30. Default value: 9999-01-01
         """
-        shop_and_services = "1" if shop_and_services == True else "0"
 
         headers = {"x-access-token": self._token}
-        data = {"language": language, "website" : website, "start_date": start_date, "end_date": end_date, "data_version": data_version, "shop_and_services": shop_and_services}
+        data = {"language": language, "website" : website, "start_date": start_date, "end_date": end_date}
         route = "/api/data/get_sexual_ads?from_id="
             
-        def get_df(info):
-            df = pd.DataFrame(info)
-            columns = ["id_ad", "data_version", "author", "language", "link", "id_page", "title", "text", "category", "first_post_date", "extract_date", "website", "whatsapp", "email", "verified_ad", "prepayment", "promoted_ad", "external_website", "reviews_website", "phone", "country", "region", "city", "place", "latitude", "longitude", "zoom"]
-            if shop_and_services: 
-                columns = columns[0: columns.index("phone") + 1]
-            df = df[columns]
-            df.set_index("id_ad", inplace = True)
-            return df
-        
         def get_total_fetch(dataframes):
             results_fetch = 0
             for df in dataframes:
@@ -184,7 +187,7 @@ class ChainBreakerClient():
             return pd.DataFrame()
         
         res = res.json()
-        df = get_df(res["ads"])
+        df = self.get_df(res["ads"])
         dataframes.append(df)
         from_id = int(res["last_id"])
         total_results = int(res["total_results"])
@@ -196,7 +199,7 @@ class ChainBreakerClient():
             if res.status_code == 401: 
                 break
             res = res.json()
-            df = get_df(res["ads"])
+            df = self.get_df(res["ads"])
             dataframes.append(df)
             from_id = int(res["last_id"])
             os.system("cls")
@@ -205,7 +208,21 @@ class ChainBreakerClient():
         
         # Join dataframes and return result.
         return pd.concat(dataframes, axis=0)
- 
+    
+    @token_required
+    def get_sexual_ads_by_id(self, ids_list: List[int], reduced_version = False):
+        reduced_version = "0" if reduced_version == False else "1"
+        data = {"ads_ids": ids_list, "reduced_version": reduced_version}
+        headers = {"x-access-token": self._token}
+        res = requests.post(self._endpoint + "/api/data/get_sexual_ads_by_id", data = data, headers = headers) #.json()["ads"]
+        if res.status_code != 200: 
+            res = res.json()
+            print(res["message"])
+            return pd.DataFrame()
+        res = res.json()
+        df = self.get_df(res["ads"], reduced = bool(int(reduced_version)))
+        return df
+
     @token_required
     def get_glossary(self, domain = ""):
         """
@@ -240,10 +257,82 @@ class ChainBreakerClient():
         df.set_index("id_keyword", inplace = True)
         return df
 
+    @token_required
+    def search_phone(self, phone):
+        headers = {"x-access-token": self._token}
+        data = {"phone": phone}
+        res = requests.post(self._endpoint + "/api/data/search_phone", data = data, headers = headers)
+        if res.status_code == 200:
+            res = res.json()
+            df = self.get_df(res["ads"])
+            print("Warning: The phone number score risk is experimental. Avoid to use it to take decisions.")
+            return df
+        else: 
+            print("Phone not found.")
+            return 404
+        
+    @token_required
+    def get_phone_score_risk(self, phone):
+        headers = {"x-access-token": self._token}
+        data = {"phone": phone}
+        res = requests.post(self._endpoint + "/api/data/get_phone_score_risk", data = data, headers = headers)
+        if res.status_code == 200:
+            score_risk = res.json()["score_risk"]
+            print("Warning: This phone number score risk is experimental. Avoid to use it to take decisions.")
+            return score_risk
+        else: 
+            print("Phone not found.")
+            return 404
+
+    @token_required
+    def get_communities(self, country = ""):
+        headers = {"x-access-token": self._token}
+        data = {"country": country}
+        res = requests.post(self._endpoint + "/api/graph/get_communities", data = data, headers = headers)
+        if res.status_code == 200:
+            communities = res.json()["communities"]
+            new_map = {}
+            for key, value in communities.items():
+                new_array = list()
+                for val in value:
+                    new_array.append(int(val))
+                new_map[int(key)] = new_array
+            return new_map
+        else: 
+            message = res.json()["message"]
+            print(message)
+            return 404
+
+    @token_required
+    def export_communities(self, country = "", export_file = ""):
+        communities = self.get_communities(country = country)
+        if communities != 404:
+            for key, ads_ids in communities.items():
+                df = self.get_sexual_ads_by_id(ads_ids)
+                df.to_excel(export_file + "/community_" + str(key) + ".xlsx")
+
+    @token_required
+    def get_labels_count(self):
+        res = requests.get(self._endpoint + "/api/graph/get_labels_count")
+        if res.status_code == 200:
+            return pd.DataFrame(res.json()["labels_count"])
+        else: 
+            message = res.json()["message"]
+            print(message)
+            return 404
+
+    @token_required
+    def get_communities_stats(self, country = ""):
+        return True
+
     @token_required
     def get_token(self):
         return self._token
 
+class ChainBreakerExpert(ChainBreakerClient):
+    def __init__(self, endpoint):
+        super().__init__(endpoint)
+
 class ChainBreakerScraper(ChainBreakerClient):
     def __init__(self, endpoint):
         super().__init__(endpoint)
@@ -260,34 +349,27 @@ class ChainBreakerScraper(ChainBreakerClient):
         data = {"url": url}
         return requests.post(self._endpoint + "/api/scraper/get_soup", data = data, headers = headers).json()["result"]
 
-
     @token_required
-    def does_ad_exist(self, id_page):
+    def does_ad_exist(self, id_page, website, country):
         """
         Get if an ad exist using th id_page.
         """
         headers = {"x-access-token": self._token}
-        data = {"id_page": id_page}
+        data = {"id_page": id_page, "website": website, "country": country}
         return requests.post(self._endpoint + "/api/scraper/does_ad_exists", data = data, headers = headers).json()["does_ad_exist"]
 
-    @token_required
-    def format_text(self, text):
-        """
-        Format a text using ChainBreaker API in order to be compatible with CB data structure.
-        """
-        headers = {"x-access-token": self._token}
-        data = {"text": text}
-        return requests.post(self._endpoint + "/api/scraper/format_text", data = data, headers = headers).json()["text"]
-
     @token_required
     def insert_ad(self, author, language, link, id_page, title, text, category,
-                  post_date, extract_date, website, whatsapp, verified_ad, prepayment, 
-                  promoted_ad, email, external_website, reviews_website, comments, country, 
-                  region, city, place): #, latitude, longitude, zoom):
+                  first_post_date, extract_date, website, phone, country, 
+                  region, city, place, email, verified_ad, prepayment, 
+                  promoted_ad, external_website, reviews_website, comments,
+                  latitude, longitude, nationality, age):
         """
         This function allow scraper to insert advertisements.
         """
         data = {}
+
+        # Ad info.
         data["author"] = author
         data["language"] = language
         data["link"] = link
@@ -295,36 +377,85 @@ class ChainBreakerScraper(ChainBreakerClient):
         data["title"] = title
         data["text"] = text
         data["category"] = category
-        data["post_date"] = post_date
+        data["first_post_date"] = first_post_date
         data["extract_date"] = extract_date
         data["website"] = website
-        data["whatsapp"] = whatsapp
-        data["verified_ad"] = verified_ad
-        data["prepayment"] = prepayment
-        data["promoted_ad"] = promoted_ad
-        data["email"] = email
-        data["external_website"] = external_website
-        data["reviews_website"] = reviews_website
-        data["comments"] = comments
+        
+        # Phone info.
+        data["phone"] = phone
+        
+        # Location info.
         data["country"] = country
         data["region"] = region
         data["city"] = city
         data["place"] = place
 
+        # Extra info.
+        data["email"] = email
+        data["verified_ad"] = verified_ad
+        data["prepayment"] = prepayment
+        data["promoted_ad"] = promoted_ad
+        data["external_website"] = external_website
+        data["reviews_website"] = reviews_website
+
+        # Comments info.
+        data["comments"] = comments
+
+        # Extra parameters.
+        data["latitude"] = latitude
+        data["longitude"] = longitude
+        data["nationality"] = nationality
+        data["age"] = age
+
+        print("Data that will be sent")
+        print(data)
+
         headers = {"x-access-token": self._token}
         res = requests.post(self._endpoint + "/api/scraper/insert_ad", data = data, headers = headers)
 
         return res.status_code
 
+    @token_required    
+    def get_image_faces(self, filepath, padding = 30):
+        #headers = {"x-access-token": self._token, "content-type": "image/jpeg"}
+        
+        # Open image and change order of b, g, r to r, g, b
+        #img = cv2.imread(filepath)
+        #b, g, r = cv2.split(img)
+        #img = cv2.merge([r, g, b])
+
+        # Encode image and send it to the app.
+        #_, img_encoded = cv2.imencode(".jpg", img)
+        #data = img_encoded.tobytes()
+        #faces = requests.post(self._endpoint + "/api/machine_learning/get_image_faces", data = data, headers = headers).json()["faces_data"]
+
+        #outputs = list()
+        #for face in faces: 
+        #    endX = int(face["endX"])
+        #    endY = int(face["endY"])
+        #    startX = int(face["startX"])
+        #    startY = int(face["startY"])
+        #    result = img[startY - padding: endY + padding, startX - padding: endX + padding]
+        #    outputs.append(result)
+        return "Sorry. Service not currently available."
+    
 class ChainBreakerAdmin(ChainBreakerScraper):
     def __init__(self, endpoint):
         super().__init__(endpoint)
 
-    def upload_gps_location(self):
-        pass
-
-    def process_image(self):
-        pass
-
-    def process_text(self):
-        pass    
\ No newline at end of file
+    @token_required        
+    def create_user(self):
+        """
+        This functions allows administrators to create new users.
+        """
+        if  self._permission == "admin":
+            name = input("User name: ")
+            email = input("User email: ")
+            permission = input("User permission: ")
+            
+            headers = {"x-access-token": self._token}
+            data = {"name": name, "email": email, "permission": permission}
+            res = requests.put(self._endpoint + "/api/user/create_user", data = data, headers = headers).json()["message"]
+            return res
+        else: 
+            print("Only administrators can execute this function.")
\ No newline at end of file
